@@ -1,6 +1,6 @@
 /* Yash: yet another shell */
 /* compparse.c: simple parser for command line completion */
-/* (C) 2007-2017 magicant */
+/* (C) 2007-2020 magicant */
 
 /* This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -75,9 +75,7 @@ static bool ctryparse_redirect(void);
 static bool cparse_for_command(void);
 static bool cparse_case_command(void);
 static bool cparse_function_definition(void);
-static wchar_t *cparse_and_expand_word(
-	tildetype_T tilde, le_contexttype_T ctxttype)
-    __attribute__((malloc,warn_unused_result));
+static bool ctryparse_word(tildetype_T tilde, le_contexttype_T ctxttype);
 static wordunit_T *cparse_word(
 	bool testfunc(wchar_t c), tildetype_T tilde, le_contexttype_T ctxttype)
     __attribute__((nonnull,malloc,warn_unused_result));
@@ -119,7 +117,7 @@ le_context_T *le_get_context(void)
     le_context_T *ctxt = xmalloc(sizeof *ctxt);
 
     cparseinfo_T parseinfo;
-    wb_init(&parseinfo.buf);
+    wb_initwithmax(&parseinfo.buf, le_main_index);
     wb_ncat_force(&parseinfo.buf, le_main_buffer.contents, le_main_index);
     parseinfo.bufindex = 0;
     parseinfo.aliaslist = NULL;
@@ -361,12 +359,10 @@ bool ctryparse_assignment(void)
 
     if (BUF[INDEX] != L'(') {
 	/* scalar variable */
-	wchar_t *value = cparse_and_expand_word(TT_MULTI, CTXT_ASSIGN);
-	if (value == NULL) {
+	if (ctryparse_word(TT_MULTI, CTXT_ASSIGN)) {
 	    empty_pwords();
 	    return true;
 	} else {
-	    free(value);
 	    return false;
 	}
     } else {
@@ -406,7 +402,6 @@ bool ctryparse_redirect(void)
     size_t index = INDEX;
     le_contexttype_T type;
     bool result;
-    wchar_t *value;
 
     while (iswdigit(BUF[index]))
 	index++;
@@ -443,12 +438,10 @@ bool ctryparse_redirect(void)
     }
 
     skip_blanks();
-    value = cparse_and_expand_word(TT_SINGLE, type);
-    if (value == NULL) {
+    if (ctryparse_word(TT_SINGLE, type)) {
 	empty_pwords();
 	return true;
     } else {
-	free(value);
 	return false;
     }
 
@@ -593,23 +586,18 @@ bool cparse_function_definition(void)
 /* Parses the word at the current position.
  * `skip_blanks' should be called before this function is called.
  * The `ctxttype' parameter is the context type of the word parsed.
- * If the word was completely parsed, the word is expanded until quote removal
- * and returned as a newly-malloced string.
- * If the parser reached the end of the input string, the return value is NULL
+ * If the word was completely parsed, the return value is false.
+ * If the parser reached the end of the input string, the return value is true
  * and the result is saved in `pi->ctxt'. However, `pi->ctxt->pwords' is NULL
  * when the preceding words need to be determined by the caller. In this case,
  * the caller must update the `pwordc' and `pwords' member. */
-wchar_t *cparse_and_expand_word(tildetype_T tilde, le_contexttype_T ctxttype)
+bool ctryparse_word(tildetype_T tilde, le_contexttype_T ctxttype)
 {
     wordunit_T *w = cparse_word(is_token_delimiter_char, tilde, ctxttype);
-    if (w == NULL) {
-	return NULL;
-    } else {
-	wchar_t *s = expand_single(w, tilde, true, false);
-	wordfree(w);
-	assert(s != NULL);
-	return s;
-    }
+    if (w == NULL)
+	return true;
+    wordfree(w);
+    return false;
 }
 
 /* Parses the word at the current position.
@@ -663,7 +651,8 @@ cparse_word:
 		if (pi->ctxt->pwords == NULL
 			&& (pi->ctxt->type & CTXT_VBRACED)) {
 		    xwcsbuf_T buf;
-		    wchar_t *prefix = expand_single(first, tilde, true, false);
+		    wchar_t *prefix =
+			expand_single(first, tilde, Q_WORD, ES_QUOTED_HARD);
 		    assert(prefix != NULL);
 		    pi->ctxt->pattern = wb_towcs(wb_catfree(
 				wb_initwith(&buf, prefix), pi->ctxt->pattern));
@@ -729,7 +718,7 @@ finish:
     pi->ctxt->type = ctxttype;
     pi->ctxt->pwordc = 0;
     pi->ctxt->pwords = NULL;
-    pi->ctxt->pattern = expand_single(first, tilde, true, false);
+    pi->ctxt->pattern = expand_single(first, tilde, Q_WORD, ES_QUOTED_HARD);
     pi->ctxt->srcindex = srcindex;
     wordfree(first);
     return NULL;
@@ -1218,7 +1207,7 @@ bool remove_braceexpand(wchar_t *s)
     bool unclosed = false;
     xwcsbuf_T buf;
 
-    wb_init(&buf);
+    wb_initwithmax(&buf, wcslen(s));
     wb_cat(&buf, s);
 
     for (size_t i = 0; i < buf.length; ) {
