@@ -276,7 +276,8 @@ bool is_update(const char *path)
 #endif
 	    );
     } else {
-	mf = xmallocs(sizeof *mf, strlen(path) + 1, sizeof *mf->mf_filename);
+	mf = xmallocs(sizeof *mf,
+		add(strlen(path), 1), sizeof *mf->mf_filename);
 	strcpy(mf->mf_filename, path);
 	ht_set(&mailfiles, mf->mf_filename, mf);
 	result = false;
