@@ -1,6 +1,6 @@
 /* Yash: yet another shell */
 /* option.c: option settings */
-/* (C) 2007-2018 magicant */
+/* (C) 2007-2020 magicant */
 
 /* This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -398,7 +398,7 @@ set_positional_parameters:
 wchar_t *normalize_option_name(const wchar_t *optname)
 {
     xwcsbuf_T result;
-    wb_init(&result);
+    wb_initwithmax(&result, wcslen(optname));
     for (const wchar_t *s = optname; *s != L'\0'; s++)
 	if (iswalnum(*s))
 	    wb_wccat(&result, towlower(*s));
@@ -530,7 +530,7 @@ void search_shell_options(const wchar_t *optname, plist_T *resultlist)
 	if (s != NULL) {
 	    if (*s == L'\0') {
 		/* exact match! */
-		pl_clear(resultlist, 0);
+		pl_truncate(resultlist, 0);
 		pl_add(resultlist, o);
 		break;
 	    } else {
@@ -556,7 +556,7 @@ void search_normal_options(const wchar_t *optname, plist_T *resultlist,
 		(shell_invocation != NULL || o->ptr != NULL)) {
 	    if (o->longopt[namelen] == L'\0') {
 		/* exact match! */
-		pl_clear(resultlist, 0);
+		pl_truncate(resultlist, 0);
 		pl_add(resultlist, o);
 		break;
 	    } else {
