@@ -1,6 +1,6 @@
 /* Yash: yet another shell */
 /* plist.h: modifiable list of pointers */
-/* (C) 2007-2010 magicant */
+/* (C) 2007-2020 magicant */
 
 /* This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -53,8 +53,10 @@ extern plist_T *pl_setmax(plist_T *list, size_t newmax)
     __attribute__((nonnull));
 extern plist_T *pl_ensuremax(plist_T *list, size_t max)
     __attribute__((nonnull));
+static inline plist_T *pl_truncate(plist_T *list, size_t newlength)
+    __attribute__((nonnull));
 extern plist_T *pl_clear(plist_T *list, void freer(void *elem))
-    __attribute__((nonnull(1)));
+    __attribute__((nonnull));
 extern plist_T *pl_replace(
 	plist_T *restrict list, size_t i, size_t ln,
 	void *const *restrict a, size_t an)
@@ -117,7 +119,7 @@ plist_T *pl_initwith(plist_T *list, void **array, size_t length)
  * Note that the list elements are not `free'd in this function. */
 void pl_destroy(plist_T *list)
 {
-    free(list->contents);
+    free(pl_toary(list));
 }
 
 /* Frees the specified pointer list and returns the contents.
@@ -126,7 +128,26 @@ void pl_destroy(plist_T *list)
  * safely cast to (char **). */
 void **pl_toary(plist_T *list)
 {
-    return list->contents;
+    void **a = list->contents;
+#ifndef NDEBUG
+    list->contents = &a[list->maxlength];
+    list->length = list->maxlength = Size_max;
+#endif
+    return a;
+}
+
+/* Shrinks the length of the pointer list to `newlength'.
+ * `newlength' must not be larger than the current length.
+ * `maxlength' of the list is not changed.
+ * It's the caller's responsibility to free the objects pointed by the removed
+ * pointers. */
+plist_T *pl_truncate(plist_T *list, size_t newlength)
+{
+#ifdef assert
+    assert(newlength <= list->length);
+#endif
+    list->contents[list->length = newlength] = NULL;
+    return list;
 }
 
 /* Inserts the first `n' elements of array `a' at offset `i' in pointer list
