@@ -740,8 +740,8 @@ bool do_assignments(const assign_T *assign, bool temp, bool export)
 
 	switch (assign->a_type) {
 	    case A_SCALAR:
-		value = expand_single_and_unescape(
-			assign->a_scalar, TT_MULTI, true, false);
+		value =
+		    expand_single(assign->a_scalar, TT_MULTI, Q_WORD, ES_NONE);
 		if (value == NULL)
 		    return false;
 		if (shopt_xtrace)
@@ -1570,6 +1570,8 @@ static void print_function(
 	const wchar_t *name, const function_T *func,
 	const wchar_t *argv0, bool readonly)
     __attribute__((nonnull));
+static char *vartype_option_string(vartype_T type)
+    __attribute__((malloc,warn_unused_result));
 #if YASH_ENABLE_ARRAY
 static int array_dump_all(const wchar_t *argv0);
 static void array_remove_elements(
@@ -1594,8 +1596,8 @@ static bool set_optind(unsigned long optind, unsigned long optsubind);
 static inline bool set_optarg(const wchar_t *value);
 static bool set_variable_single_char(const wchar_t *varname, wchar_t value)
     __attribute__((nonnull));
-static bool read_with_prompt(xwcsbuf_T *buf, xstrbuf_T *split,
-	const struct reading_option_T *ro)
+static bool read_with_prompt(
+	xwcsbuf_T *buf, xstrbuf_T *cc, const struct reading_option_T *ro)
     __attribute__((nonnull));
 static struct promptset_T promptset_for_read(
 	bool firstline, const struct reading_option_T *ro)
@@ -1605,7 +1607,7 @@ static wchar_t *read_one_line_with_prompt(
     __attribute__((malloc,warn_unused_result));
 static wchar_t *read_one_line(void)
     __attribute__((malloc,warn_unused_result));
-static bool unescape_line(const wchar_t *line, xwcsbuf_T *buf, xstrbuf_T *split)
+static bool unescape_line(const wchar_t *line, xwcsbuf_T *buf, xstrbuf_T *cc)
     __attribute__((nonnull));
 static void assign_array(const wchar_t *name, const plist_T *ranges, size_t i)
     __attribute__((nonnull));
@@ -1827,7 +1829,7 @@ void print_scalar(const wchar_t *name, bool namequote,
 {
     wchar_t *quotedvalue;
     const char *format;
-    xstrbuf_T opts;
+    char *opts;
 
     if (var->v_value != NULL)
 	quotedvalue = quote_as_word(var->v_value);
@@ -1852,16 +1854,10 @@ void print_scalar(const wchar_t *name, bool namequote,
 	case L't':
 	    assert(wcscmp(argv0, L"typeset") == 0);
 typeset:
-	    sb_init(&opts);
-	    if (var->v_type & VF_EXPORT)
-		sb_ccat(&opts, 'x');
-	    if (var->v_type & VF_READONLY)
-		sb_ccat(&opts, 'r');
-	    if (opts.length > 0)
-		sb_insert(&opts, 0, " -");
 	    format = (quotedvalue != NULL) ? "%ls%s %ls=%ls\n" : "%ls%s %ls\n";
-	    xprintf(format, argv0, opts.contents, name, quotedvalue);
-	    sb_destroy(&opts);
+	    opts = vartype_option_string(var->v_type);
+	    xprintf(format, argv0, opts, name, quotedvalue);
+	    free(opts);
 	    break;
 	default:
 	    assert(false);
@@ -1874,8 +1870,6 @@ typeset:
 void print_array(
 	const wchar_t *name, const variable_T *var, const wchar_t *argv0)
 {
-    xstrbuf_T opts;
-
     if (!xprintf("%ls=(", name))
 	return;
     if (var->v_valc > 0) {
@@ -1912,16 +1906,10 @@ void print_array(
 	    goto typeset;
 	case L't':
 	    assert(wcscmp(argv0, L"typeset") == 0);
-typeset:
-	    sb_init(&opts);
-	    if (var->v_type & VF_EXPORT)
-		sb_ccat(&opts, 'x');
-	    if (var->v_type & VF_READONLY)
-		sb_ccat(&opts, 'r');
-	    if (opts.length > 0)
-		sb_insert(&opts, 0, " -");
-	    xprintf("%ls%s %ls\n", argv0, opts.contents, name);
-	    sb_destroy(&opts);
+typeset:;
+	    char *opts = vartype_option_string(var->v_type);
+	    xprintf("%ls%s %ls\n", argv0, opts, name);
+	    free(opts);
 	    break;
 	default:
 	    assert(false);
@@ -1969,6 +1957,21 @@ end:
     free(qname);
 }
 
+/* Returns a newly malloced string that specifies options for the typeset
+ * built-in that can be used to restore the given variable type. */
+char *vartype_option_string(vartype_T type)
+{
+    xstrbuf_T opts;
+    sb_initwithmax(&opts, 4);
+    if (type & VF_EXPORT)
+	sb_ccat(&opts, 'x');
+    if (type & VF_READONLY)
+	sb_ccat(&opts, 'r');
+    if (opts.length > 0)
+	sb_insert(&opts, 0, " -");
+    return sb_tostr(&opts);
+}
+
 #if YASH_ENABLE_HELP
 const char typeset_help[] = Ngt(
 "set or print variables"
@@ -2747,12 +2750,12 @@ int read_builtin(int argc, void **argv)
     }
 
     xwcsbuf_T buf;
-    xstrbuf_T split;
+    xstrbuf_T cc;
 
     wb_init(&buf);
-    sb_init(&split);
-    if (!read_with_prompt(&buf, &split, &ro)) {
-	sb_destroy(&split);
+    sb_init(&cc);
+    if (!read_with_prompt(&buf, &cc, &ro)) {
+	sb_destroy(&cc);
 	wb_destroy(&buf);
 	return Exit_FAILURE;
     }
@@ -2776,7 +2779,7 @@ int read_builtin(int argc, void **argv)
 	if (ifs == NULL)
 	    ifs = DEFAULT_IFS;
 
-	tail = extract_fields(buf.contents, split.contents, false, ifs, &list);
+	tail = extract_fields(buf.contents, cc.contents, ifs, &list);
 	assert(list.length % 2 == 0);
     }
 
@@ -2812,22 +2815,22 @@ int read_builtin(int argc, void **argv)
     }
 
     pl_destroy(&list);
-    sb_destroy(&split);
+    sb_destroy(&cc);
     wb_destroy(&buf);
     return (!eof && yash_error_message_count == 0)
 	    ? Exit_SUCCESS : Exit_FAILURE;
 }
 
 /* Reads one line from the standard input. The result is appended to `buf' and
- * `split'. `buf' will contain no escapes or other special characters. `split'
- * is the splittability string for `buf'. The string is splittable at characters
- * that were not backslash-escaped.
+ * `cc'. `buf' will contain no escapes or other special characters. `cc' is the
+ * charcategory_T string for `buf'. It indicates whether `buf' can be split at
+ * the corresponding character when passed to `extract_fields'.
  * If `ro->raw' is true, exactly one line is read and backslashes are not
  * treated as escapes. Otherwise, line continuations cause this function to read
  * more and backslash escapes are recognized.
  * Returns false on error while reading. */
-bool read_with_prompt(xwcsbuf_T *buf, xstrbuf_T *split,
-	const struct reading_option_T *ro)
+bool read_with_prompt(
+	xwcsbuf_T *buf, xstrbuf_T *cc, const struct reading_option_T *ro)
 {
     bool firstline = true;
     bool completed = false;
@@ -2847,10 +2850,10 @@ bool read_with_prompt(xwcsbuf_T *buf, xstrbuf_T *split,
 
 	if (ro->raw) {
 	    wb_cat(buf, line);
-	    sb_ccat_repeat(split, true, wcslen(line));
+	    sb_ccat_repeat(cc, CC_SOFT_EXPANSION, wcslen(line));
 	    completed = true;
 	} else {
-	    completed = unescape_line(line, buf, split);
+	    completed = unescape_line(line, buf, cc);
 	}
 	free(line);
 
@@ -2926,11 +2929,11 @@ wchar_t *read_one_line(void)
 }
 
 /* Parses a string that may contain backslash escapes.
- * Unescaped `line' is appended to `buf' with a corresponding splittability
- * string appended to `split'. Characters are splittable iff not escaped.
+ * Unescaped `line' is appended to `buf' with a corresponding charcategory_T
+ * string appended to `cc'.
  * The result is false iff `line' ends with a line continuation.
  * The line continuation is not appended to `buf'. */
-bool unescape_line(const wchar_t *line, xwcsbuf_T *buf, xstrbuf_T *split)
+bool unescape_line(const wchar_t *line, xwcsbuf_T *buf, xstrbuf_T *cc)
 {
     for (;;) {
 	bool splitchar;
@@ -2953,7 +2956,7 @@ bool unescape_line(const wchar_t *line, xwcsbuf_T *buf, xstrbuf_T *split)
 		break;
 	}
 	wb_wccat(buf, *line);
-	sb_ccat(split, splitchar);
+	sb_ccat(cc, CC_SOFT_EXPANSION | (splitchar ? 0 : CC_QUOTED));
 	line++;
     }
 }
@@ -2978,8 +2981,9 @@ void assign_array(const wchar_t *name, const plist_T *ranges, size_t i)
 	pl_remove(&fields, 0, 1);
     }
 
-    set_array(name, fields.length, pl_toary(&fields), SCOPE_GLOBAL,
-	    shopt_allexport);
+    size_t count = fields.length;
+    void **values = pl_toary(&fields);
+    set_array(name, count, values, SCOPE_GLOBAL, shopt_allexport);
 }
 
 #if YASH_ENABLE_HELP
